function Daten = Einlesen_Nachfrage_Stadt_Stuttgart(Daten_Input, Dateiname)
% Formatiert eine Daten-Cell mit Nachfragedaten der Stadt Stuttgart um, so dass die Daten fr die Clusterung verwendet
% werden knnen.
% Folgende Bedingung muss die Excel Datei einhalten:
%
% 1. Zeile muss berschriften enthalten "Timestamp", "Object_Id" und "Plausibilisiert.qKfz.[Fzg/h]"
Ueberschrift_Zeit   = 'Timestamp';
Ueberschrift_ID     = 'Object_Id';
Ueberschrift_Q      = 'Plausibilisiert.qKfz.[Fzg/h]';

Daten = Als_Struct_schreiben(Daten_Input, Dateiname, Ueberschrift_Zeit, Ueberschrift_ID, Ueberschrift_Q);

[~, Daten_Matrix, einzelne_Obj_ID] = Daten_umformatieren(Daten);

ES_plausi.flag_q_0_zu_nan             = true;
ES_plausi.flag_q_neg_zu_nan           = true;
ES_plausi.flag_q_gleich               = true;
ES_plausi.Anzahl_Intervall_q_gleich   = 3;
[~, Daten_Matrix] = Daten_plausibilisieren(Daten_Matrix, einzelne_Obj_ID, ES_plausi);

% Auswahl der Intervallgre:
Intervallgroesse = round(mode(diff(Daten_Matrix(:,1)))*24*60); % Hufigster Wert wird verwendet
% Daten_Matrix                            = Daten_Intervallgroesse_aendern(Daten_Matrix, Intervallgroesse);


% Auswahl der IDs:
Reihenfolge_ID = einzelne_Obj_ID;
[Netzganglinie, IDs, Tageszeit, Zeit_plus_ID] = Netzganglinie_erstellen(Daten_Matrix, Reihenfolge_ID, einzelne_Obj_ID, Intervallgroesse);

Daten = Netzganglinie;


end



function Daten = Als_Struct_schreiben(Daten_Input, Dateiname, Ueberschrift_Zeit, Ueberschrift_ID, Ueberschrift_Q)

% Die Spaltensuche wird kann mit "strfind" oder "isequal" gemacht werden.
% 1: "strfind": Der Name muss enthalten, aber nicht exakt sein (z.B. berschrift = 'stamp', dann wird 'Timestamp') gefunden.
% 2: "isequal": Die berschrift muss exakt bereinstimmen.
welche_Methode_Ueberschrift = 1;

% Unter der Spalte "Timestamp" steht das Datum & Uhrzeit im Format:
Timestamp_Format    = 'dd.mm.yyyy HH:MM:SS';

% leere Struct erstellen:
Daten = struct('Zeit_Matlab', [], 'Object_Id', [], 'qKfz', []);

%% Zeit einlesen:
Zeit_Raw = Spalten_finden(Daten_Input, Ueberschrift_Zeit, Dateiname, welche_Methode_Ueberschrift);

% Umrechnen auf Matlab Zeit:
Daten.Zeit_Matlab = datenum(Zeit_Raw, Timestamp_Format);


%% Object_Id einlesen:
Daten.Object_Id = Spalten_finden(Daten_Input, Ueberschrift_ID, Dateiname, welche_Methode_Ueberschrift);


%% qKfz einlesen:
qKfz_Raw = Spalten_finden(Daten_Input, Ueberschrift_Q, Dateiname, welche_Methode_Ueberschrift);
% Als numerischer Wert schreiben:
Daten.qKfz = nan(size(qKfz_Raw)); % vordimensionieren mit NaN
% Es gibt im Beispiel "MQ Cannstatter Str 2012.xls" Zeilen welche "-" und keinen Wert enthalten.
% Diese Eintrge bleiben NaN:
idx_numerisch = cellfun(@isnumeric, qKfz_Raw);
Daten.qKfz(idx_numerisch) = [qKfz_Raw{idx_numerisch}];



end
function RAW_Data_Spalte = Spalten_finden(RAW_Data, Ueberschrift_Spalte, Excel_Datei, welche_Methode_Ueberschrift)
% In dieser Funktion wird eine Spalte in der ersten Zeile von "RAW_Data" mit der Bezeichnung "Ueberschrift_Spalte"
% gesucht. Die Werte dieser Spalte (Zeilen 2:end) werden in der Variablen "RAW_Data_Spalte" zurckgegeben.

% Die Spaltensuche wird kann mit "strfind" oder "isequal" gemacht werden.
% 1: "strfind": Der Name muss enthalten, aber nicht exakt sein (z.B. berschrift = 'stamp', dann wird 'Timestamp') gefunden.
% 2: "isequal": Die berschrift muss exakt bereinstimmen.

switch welche_Methode_Ueberschrift,
    case 1, % 1: "strfind": Der Name muss enthalten, aber nicht exakt sein (z.B. berschrift = 'stamp', dann wird 'Timestamp') gefunden.
        Ueberschriften = RAW_Data(1,:);
        idx_char = cellfun(@ischar, RAW_Data(1,:)); % Es kann vorkommen, dass manche berschriften numerisch sind.
        [Ueberschriften{~idx_char}] = deal('');
        Spalte_Nr = find(cellfun(@any, strfind(Ueberschriften, Ueberschrift_Spalte)));
    case 2, % 2: "isequal": Die berschrift muss exakt bereinstimmen.
        Spalte_Nr = find(cellfun(@(x) isequal(x, Ueberschrift_Spalte), RAW_Data(1,:)));
end

% Prfen auf Fehler:
if isempty(Spalte_Nr),
    error(['In der Datei "',Excel_Datei,'" wurde keine Spalte mit der Bezeichnung"',Ueberschrift_Spalte,'" gefunden.'])
end
if length(Spalte_Nr) > 1,
    error(['In der Datei "',Excel_Datei,'" wurde mehrere Spalten mit der Bezeichnung"',Ueberschrift_Spalte,'" gefunden.'])
end    
RAW_Data_Spalte = RAW_Data(2:end, Spalte_Nr);

end



function [Daten_mehrdimensional, Daten_Matrix, einzelne_Obj_ID] = Daten_umformatieren(Daten)
% Fasst verschiedne Excel-Dateien zusammen.
% Auch die Zeiten werden zusammengefasst.
%
%
% Daten_Matrix ist ...
%   1. Spalte  | 2.-n.te Spalte  (je nach Anzahl Detektoren (einzelne_Obj_ID))
%       Zeit   |   Zhlwert eines Detektors


Felder = fieldnames(Daten);

% Verschiedene Einzeldateien zusammenfassen:
for cnt_F = 1 : length(Felder),
    akt_Feld = Felder{cnt_F};
    Daten_alle.(akt_Feld) = vertcat(Daten.(akt_Feld));
end

[einzelne_Zeiten, ~, idx_Zeit] = unique(Daten_alle.Zeit_Matlab);

%% Die Zeiten werden auf 15 Mintuen gerundet:
% Rohdaten:
% 01-Jan-2012 22:30:00
% 01-Jan-2012 22:45:00
% 01-Jan-2012 23:00:00
% 01-Jan-2012 23:15:00
% 01-Jan-2012 23:30:00
% 01-Jan-2012 23:45:00
% 01-Jan-2012 23:59:59 <= Dieses Zeitintervall soll besser um 02-Jan-2012 00:00:00 beginnen.
% 02-Jan-2012 00:15:00
% 02-Jan-2012 00:30:00
% 02-Jan-2012 00:45:00
%
% Das wird bei Intervallgre ndern bentigt !!!

func_Runden_auf_15_Minuten = @(x) round(x * 24 * 60/15) / (60/15) / 24;
einzelne_Zeiten = func_Runden_auf_15_Minuten(einzelne_Zeiten);

[einzelne_Obj_ID, ~, idx_Obj_ID] = unique(Daten_alle.Object_Id);

% Es wird eine Matrix mit ...
%   1. Spalte  | 2.-n.te Spalte  (je nach Anzahl Detektoren (einzelne_Obj_ID))
%       Zeit   |   Zhlwert eines Detektors
Daten_Matrix = nan(length(einzelne_Zeiten), 1 + length(einzelne_Obj_ID)); % vordimensionieren

Daten_Matrix(:, 1) = einzelne_Zeiten;

Anz_Obj_ID = length(einzelne_Obj_ID);

for cnt_Obj_ID = 1 : Anz_Obj_ID,
    idx_akt_Obj_ID = idx_Obj_ID == cnt_Obj_ID;
    % Nach der akt_Obj filtern:
    Daten_akt_Obj       = Struct_Felder_filtern(Daten_alle, idx_akt_Obj_ID);
    idx_Zeit_akt_Obj    = idx_Zeit(idx_akt_Obj_ID);
    
    Daten_Matrix(idx_Zeit_akt_Obj, 1 + cnt_Obj_ID) = Daten_akt_Obj.qKfz;
end

%% Als Struct schreiben:
Daten_Struct.Zeit_Matlab = Daten_Matrix(:,1);

for cnt_Obj_ID = 1 : Anz_Obj_ID,
    text_akt_Obj_ID = strrep(einzelne_Obj_ID{cnt_Obj_ID}, '.', '_'); % es drfen keine Punkte in der Feldnamen dabei sein.
    akt_Feld = ['qKfz_ID_',text_akt_Obj_ID];
    Daten_Struct.(akt_Feld) = Daten_Matrix(:, 1 + cnt_Obj_ID);
end

% Umschreiben der Daten, Struct mehrdimensional:
Daten_mehrdimensional = Struct_von_eindimensional_zu_mehrdimensional(Daten_Struct);

end

function [Daten_mehrdimensional, Daten_Matrix] = Daten_plausibilisieren(Daten_Matrix, einzelne_Obj_ID, ES)
% Untersucht die Daten auf Fehler und setzt die Daten an dieser Stelle auf NaN.
% Vorhergige Funktionen:
% Daten_mehrdimensional                 = Einlesen_Nachfrage_Stadt_Stuttgart(Excel_Datei)
%
% Geprft wird: 
%   - ob die Daten 0 =>            Strung oder Ausfall => diese Daten sollten nicht in die Clusterung eingehen. => NaN
%   - negativ sind                              Ausfall => diese Daten sollten nicht in die Clusterung eingehen. => NaN
%   - 3 ZI in Folge die selben Werte auftreten  Ausfall => diese Daten sollten nicht in die Clusterung eingehen. => NaN

Anzahl_IDs = length(unique(einzelne_Obj_ID));

ES_Standard.flag_q_0_zu_nan             = true;
ES_Standard.flag_q_neg_zu_nan           = true;
ES_Standard.flag_q_gleich               = true;
ES_Standard.Anzahl_Intervall_q_gleich   = 3;

if nargin < 3 || isempty(ES) || ~isstruct(ES),
    ES = struct();
end

% Nicht vorhandene Felder von "ES_Standard" zu "ES" hinzufgen.
ES = Struct_nicht_vorhandene_Felder_uebernehmen(ES, ES_Standard);

for cnt_ID = 1 : Anzahl_IDs,
    akt_ID      = einzelne_Obj_ID{cnt_ID};
    akt_Spalte  = cnt_ID + 1;
    
    %%   - ob die Daten 0 => Strung oder Ausfall => diese Daten sollten nicht in die Clusterung eingehen.
    if ES.flag_q_0_zu_nan,
        idx_Null = Daten_Matrix(:, akt_Spalte) == 0;
        Daten_Matrix(idx_Null, akt_Spalte) = nan;
    end
    
    %%   - negativ sind                   Ausfall => diese Daten sollten nicht in die Clusterung eingehen.
    if ES.flag_q_neg_zu_nan,
        idx_negativ = Daten_Matrix(:, akt_Spalte) < 0;
        Daten_Matrix(idx_negativ, akt_Spalte) = nan;
    end
    
    %%   - 3 ZI in Folge die selben Werte auftreten  Ausfall => diese Daten sollten nicht in die Clusterung eingehen. => NaN
    if ES.flag_q_gleich,
        % Diese Funktion gibt einen Vektor zurck, welcher zurckgibt in welchen Zeilen die Folgewerte gleich sind.
        % "Anzahl_gleiche_Folgeintervalle" gibt zurck, wie viele Folgeintervalle identisch sind.
        % 
        Vektor = Daten_Matrix(:, akt_Spalte);
        [~, Anzahl_gleiche_Folgeintervalle] = Vektor_gleiche_Folgewerte(Vektor);
        
        Zeilen_NR_NaN_setzten = find(Anzahl_gleiche_Folgeintervalle >= ES.Anzahl_Intervall_q_gleich);
        
        for cnt_Z = 1 : length(Zeilen_NR_NaN_setzten),
            akt_Zeile = Zeilen_NR_NaN_setzten(cnt_Z);
            Anzahl_der_Zeilen_NaN_setzten = Anzahl_gleiche_Folgeintervalle(akt_Zeile);
            
            flag_mit_pruefung = true;
            if flag_mit_pruefung,
                Anzahl_unterschiedlicher_Werte = length(unique(Daten_Matrix(akt_Zeile : akt_Zeile + Anzahl_der_Zeilen_NaN_setzten - 1, akt_Spalte)));
                if Anzahl_unterschiedlicher_Werte == 1,
                    Daten_Matrix(akt_Zeile : akt_Zeile + Anzahl_der_Zeilen_NaN_setzten - 1, akt_Spalte) = nan;
                else
                    error('Ein Fehler ist aufgetreten.')
                end
            else
                Daten_Matrix(akt_Zeile : akt_Zeile + Anzahl_der_Zeilen_NaN_setzten - 1, akt_Spalte) = nan;
            end

        end
    end
    
    
end

%% Als Struct schreiben:
Daten_Struct.Zeit_Matlab = Daten_Matrix(:,1);

for cnt_Obj_ID = 1 : Anzahl_IDs,
    text_akt_Obj_ID = strrep(einzelne_Obj_ID{cnt_Obj_ID}, '.', '_'); % es drfen keine Punkte in der Feldnamen dabei sein.
    akt_Feld = ['qKfz_ID_',text_akt_Obj_ID];
    Daten_Struct.(akt_Feld) = Daten_Matrix(:, 1 + cnt_Obj_ID);
end

% Umschreiben der Daten, Struct mehrdimensional:
Daten_mehrdimensional = Struct_von_eindimensional_zu_mehrdimensional(Daten_Struct);

end % MAIN function
function Daten_Matrix_neues_Intervall = Daten_Intervallgroesse_aendern(Daten_Matrix, Intervallgroesse)

if nargin < 2 || isempty(Daten_Matrix) || isempty(Intervallgroesse),
    warning('Falsche oder keine Eingnge => keine nderung der Intervallgre.')
    Daten_Matrix_neues_Intervall = Daten_Matrix;
    return
end

% Prfen, ob die aktuelle Intervallgre bereits der geforderten Intervallgre entspricht:
aktuelle_Intervallgroesse = round(mode(diff(Daten_Matrix(:,1)))*24*60);
if aktuelle_Intervallgroesse == Intervallgroesse,
    Daten_Matrix_neues_Intervall = Daten_Matrix;
    return
end

Zeiten_ursp_Intervalle = Daten_Matrix(:,1);

Zeiten_neue_Intervalle = floor(min(Daten_Matrix(:,1))) : Intervallgroesse/60/24 : ceil(max(Daten_Matrix(:,1)));


% Die ursprnglichen Zeitintervelle beginnen am Tageswechsel um 23:59:59 Uhr.
% Das sollte aber in der Funktion "Daten_umformatieren" behoben werden sein.
% Dort werden die Intervalle auf 15 Minuten genau gerundet.


Daten_Matrix_neues_Intervall        = nan(length(Zeiten_neue_Intervalle), size(Daten_Matrix, 2)); % vordimensionieren
Daten_Matrix_neues_Intervall(:, 1)  = Zeiten_neue_Intervalle;

for cnt_S = 2 : size(Daten_Matrix, 2),
    Vektor = Daten_Matrix(:, cnt_S);
    wie_viele_Elemente_zusammenpacken{1} = Zeiten_ursp_Intervalle;
    wie_viele_Elemente_zusammenpacken{2} = Zeiten_neue_Intervalle;
    Funktion_zusammenfassen = @mean_ohne_nan;
    Daten_Matrix_neues_Intervall(:, cnt_S) = Intervallgroesse_aendern(Vektor, wie_viele_Elemente_zusammenpacken, Funktion_zusammenfassen);
end
end


function [Netzganglinie, IDs, Tageszeit, Zeit_plus_ID] = Netzganglinie_erstellen(Daten_Matrix, Reihenfolge_ID, einzelne_Obj_ID, Intervallgroesse)
% Erstellt eine Netzganglinie mit den Daten der Stadt Stuttgart.
%
%   Eingang: ----------------------------------------------
% - Daten_Matrix
%       Daten_Matrix ist ...
%           1. Spalte  | 2.-n.te Spalte  (je nach Anzahl Detektoren (einzelne_Obj_ID))
%               Zeit   |   Zhlwert eines Detektors
% - Reihenfolge_ID
%       Die Reihenfolge in der die IDs zur Netzganglinie zusammengesetzt werden soll.
% - einzelne_Obj_ID
%       Alle ID, welche in "Daten_Matrix" vorkommen. 
%       Die Zhlwerte von einzelne_Obj_ID{1} mssen in der Spalte Daten_Matrix(:, 1 + 1) stehen. 
%       Die Zhlwerte von einzelne_Obj_ID{n} mssen in der Spalte Daten_Matrix(:, 1 + n) stehen.
% - Intervallgroesse
%       In Minuten [min]
%
%   Rckgabe: ----------------------------------------------
% - Netzganglinie   
%       n x m Matrix mit n = Anzahl der Tage, m = Anzahl der Zeitintervalle eines Tages * Anzahl Objekte (Detektoren) + 1 (erste Spalte steht der Tage)
%       1. Spalte: Zeit im Matlab Format (ganze Tage)
%       2.- m-te Spalte: Verkehrsstrken an diesem Tag, geordnet zuerst nach den Objekten (Detektoren), dann nach der
%       Zeit 
%           => 2.  Spalte ID 123 Zeit 00:00,  3. Spalte ID 123 Zeit 01:00,  4. Spalte ID 123 Zeit 02:00
%              25. Spalte ID 123 Zeit 23:00, 26. Spalte ID 777 Zeit 00:00, 27. Spalte ID 777 Zeit 01:00
% - IDs
%       1 x (m-1)   Cell mit der     ID    fr jede Spalte von "Netzganglinie" in der eine Verkehrsstrke enthalten ist.
% - Tageszeit
%       1 x (m-1) Vektor mit der Tageszeit fr jede Spalte von "Netzganglinie" in der eine Verkehrsstrke enthalten ist.
% - Zeit_plus_ID
%       1 x (m-1)   Cell mit einer zusammengefassten Bezeichnung fr jede Spalte von "Netzganglinie" in der eine Verkehrsstrke enthalten ist.
%       z.B. {'00:00 ID: 123', '01:00 ID: 123', ..., '23:00 ID: 123', '00:00 ID: 777', '01:00 ID: 777', ..., '23:00 ID: 777'}


Zeit = Daten_Matrix(:,1);

%% Alle Tage welche vorkommen:
[Alle_Tage, ~, idx_Tag] = unique(floor(Zeit));

%% TOD Zeiten berechnen:
flag_nur_tod        = true;
flag_addiere_10ms   = true; % Zur Zeit wird jeweils 10ms hinzuaddiert, damit Rundungsfehler vermieden werden knnen.
[idx_toddow, toddow_Intervalle] = toddow( Zeit, Intervallgroesse, flag_nur_tod, flag_addiere_10ms );

Anzahl_Tage                     = max(idx_Tag);
Anzahl_Zeitintervalle_ein_Tag   = max(idx_toddow);

Anzahl_Obj_in_NGLM = length(Reihenfolge_ID); % Anzahl der Objekte (Detektoren) in der NetzganglinienMatrix

% Hier wird die Reihenfolge der Spalten festgelegt, wie die Netzganglinien zusammengesetzt wird.
[~, Reihenfolge_Spalten] = ismember(Reihenfolge_ID, einzelne_Obj_ID);

%% GMatrix == GanglinienMatrix (ohne Zeit in der ersten Spalte).
% Es wird fr jedes Objekt (Detektor) eine seperate GanglinienMatrix gebildet.
GMatrix = cell(1, Anzahl_Obj_in_NGLM); % vordimensionieren
for cnt_Spalte = 1 : Anzahl_Obj_in_NGLM,
    akt_Spalte = Reihenfolge_Spalten(cnt_Spalte) + 1; % +1, da in der ersten Spalte von "Daten_Matrix" die Zeit steht.
    
    GMatrix{cnt_Spalte}       = nan(Anzahl_Tage, Anzahl_Zeitintervalle_ein_Tag); % vordimensionieren
    
    welche_Methode = 2; % Methode 2 ist schneller. Ergenis natrlich identisch :).
    switch welche_Methode,
        case 1,
            % Werte in die Matrix eintragen:
            % Fr jeden Tag machen:
            for cnt_Tag = 1 : length(Alle_Tage),
                idx_Zeile_akt_Tag = idx_Tag == cnt_Tag;
                GMatrix{cnt_Spalte}(cnt_Tag, idx_toddow(idx_Zeile_akt_Tag)) = Daten_Matrix(idx_Zeile_akt_Tag, akt_Spalte);
            end
            
        case 2,
            % Werte in die Matrix eintragen:
            % Fr jedes Element machen:
            for cnt_i = 1 : size(Daten_Matrix, 1),
                GMatrix{cnt_Spalte}(idx_Tag(cnt_i), idx_toddow(cnt_i)) = Daten_Matrix(cnt_i, akt_Spalte);
            end
    end
end

%% Jetzt die einzelnen GMatrix (= GanglinienMatrix) zur NetzGangLinienMatrix zusammenpacken:
Netzganglinie = nan(Anzahl_Tage, Anzahl_Zeitintervalle_ein_Tag * Anzahl_Obj_in_NGLM + 1); % vordimensionieren

% Erste GanglinienMatrix bergeben:
Netzganglinie(:, 1)     = Alle_Tage;
Netzganglinie(:, 2:end) = horzcat(GMatrix{:});


%% Weitere Ausgnge:

if nargin > 1,
    %% - IDs
    %       1 x (m-1)   Cell mit der     ID    fr jede Spalte von "Netzganglinie" in der eine Verkehrsstrke enthalten ist.
    
    welcher_Vektor = 1;
    %   1: Vektor der Form: [1,1,1,1,               2,2,2,2,                3,3,3,3, ...,           max_Wert, max_Wert, max_Wert, max_Wert] bei Hauefigkeit = 4;
    %   2: Vektor der Form: [1,2,3,...,max_Wert,    1,2,3,...,max_Wert,     1,2,3,...,max_Wert,     1,2,3,...,max_Wert                    ] bei Hauefigkeit = 4;    
    Haeufigkeit = Anzahl_Zeitintervalle_ein_Tag;
    max_Wert    = Anzahl_Obj_in_NGLM;
    idx_IDs     = idx_Vektor_generieren(welcher_Vektor, Haeufigkeit, max_Wert);
    IDs         = cell(1, length(idx_IDs)); % Als Spaltenvektor vordimensionieren
    IDs(1:end)  = Reihenfolge_ID(idx_IDs);
end

if nargin > 2,
%% - Tageszeit
    %       1 x (m-1) Vektor mit der Tageszeit fr jede Spalte von "Netzganglinie" in der eine Verkehrsstrke enthalten ist.
    
    welcher_Vektor      = 2;
    %   1: Vektor der Form: [1,1,1,1,               2,2,2,2,                3,3,3,3, ...,           max_Wert, max_Wert, max_Wert, max_Wert] bei Hauefigkeit = 4;
    %   2: Vektor der Form: [1,2,3,...,max_Wert,    1,2,3,...,max_Wert,     1,2,3,...,max_Wert,     1,2,3,...,max_Wert                    ] bei Hauefigkeit = 4;    
    Haeufigkeit         = Anzahl_Obj_in_NGLM;
    max_Wert            = Anzahl_Zeitintervalle_ein_Tag;
    idx_Tageszeit       = idx_Vektor_generieren(welcher_Vektor, Haeufigkeit, max_Wert);
    Tageszeit           = nan(1, length(idx_Tageszeit)); % Als Spaltenvektor vordimensionieren
    Tageszeit(1:end)    = toddow_Intervalle(idx_Tageszeit);
end

if nargin > 3,
    %% - Zeit_plus_ID
    %       1 x (m-1)   Cell mit einer zusammengefassten Bezeichnung fr jede Spalte von "Netzganglinie" in der eine Verkehrsstrke enthalten ist.
    %       z.B. {'00:00 ID: 123', '01:00 ID: 123', ..., '23:00 ID: 123', '00:00 ID: 777', '01:00 ID: 777', ..., '23:00 ID: 777'}
    % Zeit_plus_ID = cell(1, Anzahl_Zeitintervalle_ein_Tag * Anzahl_Obj_in_NGLM); % Als Spaltenvektor vordimensionieren
    Zeit_plus_ID = cellfun(@(x,y) [datestr(x, 'HH:MM'),32,'ID:',32,y], num2cell(Tageszeit), IDs, 'UniformOutput', false);
    
    
end

end


































