function [FerienTage, FerienTage_Name] = Ferien_eines_Jahres(Jahr, Bundesland)

% Die Funktion laedt die Ferien von dem gewaehlten und dem naechsten Jahr,
% ersetzt dann die Weihnachtsferien des momentanen Jahres, die an erster
% Stelle der Ferientabelle stehen und fuegt die Weihnachtsferien des naechsten Jahres ans
% Ende hinzu.
% Der Grund dafuer ist, dass der alte Code die Ferien von Schulferien.org
% geladen hat. Die Seite ist so aufgebaut, dass die Weihnachtsferien angezeigt werden, 
% die am Ende des Jahres anstehen und ins naechste Jahr uebergehen.
% Bei der neuen Seite werden die Weihnachtsferien, die letztes Jahr
% begannen und ins diese Jahr uebergehen angezeigt.

Jahr_str = num2str(Jahr);

% HTML_Text = urlread(['http://www.schulferien.org/Schulferien_nach_Jahren/',Jahr_str,'/schulferien_',Jahr_str,'.html']);
HTML_Text = webread(['http://schulferien.kfz-auskunft.de/schulferien_',Jahr_str,'.html']);

HTML_Text_Cell = String_teilen(HTML_Text,'<tr valign="top">')';

Bundesland_HTML = Bundesland__wie_in_der_URL(Bundesland);

idx_Start_Bundesland = strfind(HTML_Text, Bundesland_HTML);

% Reihenfolgedb der Ferien:
Ferienreihenfolge = {'Weihnachten'; 'Winterferien'; 'Osterferien'; 'Pfingstferien'; 'Sommerferien'; 'Herbstferien'};

[~, Zeile_Bundesland] = histc(idx_Start_Bundesland, cumsum(cellfun(@numel, HTML_Text_Cell)));

Zeile_Bundesland = unique(Zeile_Bundesland);

% Die Ferientabelle befindet sich immer eine Cell nach dem Bundesland:
Ferien_Bundesland = HTML_Text_Cell{Zeile_Bundesland(end) + 1};

% Wenn die Zeile zu Ende ist kann abgebrochen werden:
% Loescht alles was hiter '</tr>' kommt
idx_Zeilenende = strfind(Ferien_Bundesland, '</tr>');

if any(idx_Zeilenende),
    Ferien_Bundesland(idx_Zeilenende:end) = [];
end

Trennzeichen = {'<td align="center">'};

Einzelne_Ferien = regexp(Ferien_Bundesland, Trennzeichen, 'split'); 

Einzelne_Ferien{1}(1) = []; % Strings{1}{1} beinhaltet noch den Bundeslandnamen;
% </td> aus Einzelne_Ferien lschen:
for cnt_F = 1 : length(Einzelne_Ferien{1}),
    idx_loeschen = strfind(Einzelne_Ferien{1}{cnt_F}, '</td>');
    if any(idx_loeschen),
        Einzelne_Ferien{1}{cnt_F}(idx_loeschen:idx_loeschen + 4) = [];
    end
end


% 1. Spalte: von Tag
% 2. Spalte: bis Tag
% 3. Spalte: zustzlicher und/oder einzelner Tag
FerienZeitraum_Str = cell(length(Einzelne_Ferien{1}), 4); % vordimensionieren

for cnt_F = 1 : length(Einzelne_Ferien{1}), 
    % Falls Anmerkungen vorkommen, diese rauslschen:
    if any(strfind(Einzelne_Ferien{1}{cnt_F},'<SUP>')),
        idx_Beginn = strfind(Einzelne_Ferien{1}{cnt_F},'<SUP>');
        idx_Ende   = strfind(Einzelne_Ferien{1}{cnt_F},'</SUP>') + 5;
        Einzelne_Ferien{1}{cnt_F}(idx_Beginn:idx_Ende) = [];
    end
    
    Einzelne_Ferien{1}{cnt_F} = strtrim(Einzelne_Ferien{1}{cnt_F});
    
    if length(Einzelne_Ferien{1}{cnt_F}) < 6 || Einzelne_Ferien{1}{cnt_F}(1) == '-', % length 6 wegen xx.xx.
        % Diese Ferien gibt es in dem Bundesland nicht.
    else
        % Verschiedene Flle ber die Lnge der Strings:
        % 1. Standard: dd.mm.-dd.mm. ========================== CASE 13
        % 2. Zwei Tage: dd.mm+dd.mm. ========================== CASE 13
        % 3. Ein Tag: dd.mm. ================================== CASE  6
        % 4. Standard plus extra Tag: dd.mm.-dd.mm./dd.mm. ==== CASE 20
        % 5. Extra Tag plus Standard: dd.mm.+dd.mm.-dd.mm. ==== CASE 20
        % 6. Drei einzelne Tage: dd.mm.+dd.mm.+dd.mm. ========= CASE 20
        switch length(Einzelne_Ferien{1}{cnt_F}),
            
            case 13, % 1. und 2.
                
                % 1. Standard: dd.mm.-dd.mm.
                if Einzelne_Ferien{1}{cnt_F}(7) == '-';
                FerienZeitraum_Str{cnt_F, 1} = Einzelne_Ferien{1}{cnt_F}( 1 : 6);
                FerienZeitraum_Str{cnt_F, 2} = Einzelne_Ferien{1}{cnt_F}( 8 :13);
                end
                % 2. Zwei Tage: dd.mm.+dd.mm.
                if Einzelne_Ferien{1}{cnt_F}(7) == '+';
                FerienZeitraum_Str{cnt_F, 1} = Einzelne_Ferien{1}{cnt_F}( 1 : 6); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 2} = Einzelne_Ferien{1}{cnt_F}( 1 : 6); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 3} = Einzelne_Ferien{1}{cnt_F}( 8 : 13); % die zustzliche Zeit in Spalte 3    
                end
                
            case  6, % 3. Ein Tag: dd.mm.
                FerienZeitraum_Str{cnt_F, 3} = Einzelne_Ferien{1}{cnt_F}( 1 : 6);
                
            case 20, % 4. und 5. und 6.
                % 4. Standard plus extra Tag: dd.mm.-dd.mm.,dd.mm.
                if (Einzelne_Ferien{1}{cnt_F}(7) == '-' && Einzelne_Ferien{1}{cnt_F}(14) == ',') || (Einzelne_Ferien{1}{cnt_F}(7) == '-' && Einzelne_Ferien{1}{cnt_F}(14) == '+')
                FerienZeitraum_Str{cnt_F, 1} = Einzelne_Ferien{1}{cnt_F}( 1 : 6); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 2} = Einzelne_Ferien{1}{cnt_F}( 8 :13); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 3} = Einzelne_Ferien{1}{cnt_F}(15 :20); % die zustzliche Zeit in Spalte 3    
                end
                % 5. Extra Tag plus Standard: dd.mm.,dd.mm.-dd.mm.
                if (Einzelne_Ferien{1}{cnt_F}(7) == ',' && Einzelne_Ferien{1}{cnt_F}(14) == '-') || ((Einzelne_Ferien{1}{cnt_F}(7) == '+' && Einzelne_Ferien{1}{cnt_F}(14) == '-'))
                FerienZeitraum_Str{cnt_F, 1} = Einzelne_Ferien{1}{cnt_F}( 8 :13); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 2} = Einzelne_Ferien{1}{cnt_F}(15 :20); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 3} = Einzelne_Ferien{1}{cnt_F}( 1 : 6); % die zustzliche Zeit in Spalte 3    
                end
                % 6. Einzelne Tage: dd.mm.+dd.mm.+dd.mm.
                if Einzelne_Ferien{1}{cnt_F}(7) == '+' && Einzelne_Ferien{1}{cnt_F}(14) == '+'
                FerienZeitraum_Str{cnt_F, 1} = Einzelne_Ferien{1}{cnt_F}( 1: 6); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 2} = Einzelne_Ferien{1}{cnt_F}( 1: 6); % gleiche Zeit bei Spalte 1 & 2
                FerienZeitraum_Str{cnt_F, 3} = Einzelne_Ferien{1}{cnt_F}( 8:13); % die zustzliche Zeit in Spalte 3                
                FerienZeitraum_Str{cnt_F, 4} = Einzelne_Ferien{1}{cnt_F}(15:20); % die zustzliche Zeit in Spalte 3     
                end
            case 27
                % dd.mm.,dd.mm.,dd.mm.-dd.mm.
                FerienZeitraum_Str{cnt_F, 1} = Einzelne_Ferien{1}{cnt_F}(15:20);
                FerienZeitraum_Str{cnt_F, 2} = Einzelne_Ferien{1}{cnt_F}(22:27);
                FerienZeitraum_Str{cnt_F, 3} = Einzelne_Ferien{1}{cnt_F}( 1: 6);
                FerienZeitraum_Str{cnt_F, 4} = Einzelne_Ferien{1}{cnt_F}( 8:13);
            otherwise
                error('Unbekanntes Format')
        end
    end
end
% Zeilen mit keinen Ferien ausfiltern:
idx_Ferien = any(cellfun(@any, FerienZeitraum_Str),2);
FerienZeitraum_Str(~idx_Ferien, :) = [];
Ferienreihenfolge(~idx_Ferien) = [];


% In eine Matlab-Zeit umrechnen:
FerienTage_cell = cell(size(FerienZeitraum_Str,1), 2); % vordimensionieren
for cnt_F = 1 : size(FerienZeitraum_Str,1),
    if any(FerienZeitraum_Str{cnt_F, 1}) && any(FerienZeitraum_Str{cnt_F, 2}),
        % Fall wenn FerienZeitraum_Str{cnt_F, 1} und FerienZeitraum_Str{cnt_F, 2} besetzt sind, d.h. es einen Zeitraum gibt: 
        FerienEnde = datenum([FerienZeitraum_Str{cnt_F,2},Jahr_str], 'dd.mm.yyyy');
        if isequal(Ferienreihenfolge{cnt_F}, 'Weihnachten') && isequal(FerienZeitraum_Str{cnt_F,2}(4:5), '01'),
            letztes_Jahr_str = num2str(str2double(Jahr_str)-1); % Die Weihnachtsferien von letzes Jahr.
            FerienBeginn = datenum([FerienZeitraum_Str{cnt_F,1},letztes_Jahr_str], 'dd.mm.yyyy');
        else
            FerienBeginn = datenum([FerienZeitraum_Str{cnt_F,1},Jahr_str], 'dd.mm.yyyy');
        end
        FerienTage_cell{cnt_F,1} = ctranspose(FerienBeginn : FerienEnde);
        FerienTage_cell{cnt_F,2} = cell(size(FerienTage_cell{cnt_F,1})); % vordimensionieren
        [FerienTage_cell{cnt_F,2}{:}] = deal(Ferienreihenfolge{cnt_F});
    end
    
    if any(FerienZeitraum_Str{cnt_F, 3}),
        if isequal(Ferienreihenfolge{cnt_F}, 'Weihnachten') && isequal(FerienZeitraum_Str{cnt_F,3}(4:5), '12'),
            letztes_Jahr_str = num2str(str2double(Jahr_str)-1); % Die Weihnachtsferien reichen ins nchste Jahr.
            FerienTag = datenum([FerienZeitraum_Str{cnt_F,3},letztes_Jahr_str], 'dd.mm.yyyy');
        else
            FerienTag = datenum([FerienZeitraum_Str{cnt_F,3},Jahr_str], 'dd.mm.yyyy');
        end
        FerienTage_cell{cnt_F,1}(end + 1,1) = FerienTag;
        FerienTage_cell{cnt_F,2}{end + 1,1} = Ferienreihenfolge{cnt_F};
    end
    if any(FerienZeitraum_Str{cnt_F, 4}),
        if isequal(Ferienreihenfolge{cnt_F}, 'Weihnachten') && isequal(FerienZeitraum_Str{cnt_F,4}(4:5), '12'),
            letztes_Jahr_str = num2str(str2double(Jahr_str)-1); % Die Weihnachtsferien reichen ins nchste Jahr.
            FerienTag = datenum([FerienZeitraum_Str{cnt_F,4},letztes_Jahr_str], 'dd.mm.yyyy');
        else
            FerienTag = datenum([FerienZeitraum_Str{cnt_F,4},Jahr_str], 'dd.mm.yyyy');
        end
        FerienTage_cell{cnt_F,1}(end + 1,1) = FerienTag;
        FerienTage_cell{cnt_F,2}{end + 1,1} = Ferienreihenfolge{cnt_F};
    end    
end





% Alle Ferientage
FerienTage = vertcat(FerienTage_cell{:,1});
FerienTage_Name = vertcat(FerienTage_cell{:,2});


end % function 
