function Struct_AUS = Struct_von_eindimensional_zu_mehrdimensional(Struct_IN)
% Schreibt die Felder einer Struct als mehrdimensionale Struct:
% Bedingung:  
%   - Es drfen keine Unterfelder vorhanden sein.
%
% Falls die Felder unterschiedliche Lngen haben, bleiben die letzten Eintrge von den Feldern mit krzerer Lnge leer.
% Es werden immer Spaltenvektoren (n x 1) zurckgegeben. Alle Zeilenvektoren oder Matrizen werden in Zeilenvektoren
% umgeschrieben.
%
%                               JL 14.10.2013
%
% % Beispiel:
% Struct_IN.A = [1; 2; 3; 4];
% Struct_IN.B = {'Eins', 'Zwei', 'Drei', 'Vier', 'Fnf'};
% Struct_AUS = Struct_von_eindimensional_zu_mehrdimensional(Struct_IN)

% Eingang prfen:
if nargin == 0 || isempty(Struct_IN) || ~isstruct(Struct_IN),
    error('Es gibt keinen Eingang oder der Eingang ist keine Struct.')
end

% % Prfen, ob alle Elemente von "Struct_IN" die gleiche Lnge haben:
% Anzahl_Element_pro_Feld = structfun(@length, Struct_IN);
% if length(unique(Anzahl_Element_pro_Feld)) > 1,
%     error('Die Felder von "Struct_IN" haben eine unterschiedliche Lnge und knnen nicht zusammengefasst werden.')
% end

% Prfen, ob es "UnterStructs" gibt:
if any(structfun(@isstruct, Struct_IN)),
    error('Die Struct enthlt "UnterStructs" (Felder, welche wieder Structs sind). Diese Struct werden nicht untersttzt.')
end

Anzahl_Element_pro_Feld = structfun(@length, Struct_IN);
Anzahl_Elemete = max(Anzahl_Element_pro_Feld);

Felder = fieldnames(Struct_IN);

Struct_AUS(Anzahl_Elemete, 1).(Felder{1}) = []; % Struct vordimensionieren

welche_Variante = 2;
switch welche_Variante,
    
    case 1, %  ALT: Umweg ber cell2struct.
        for cnt_F = 1 : length(Felder),
            akt_Feld = Felder{cnt_F};
            % ndern von einem Feld:
            % Feld als Cell schreiben:
            if iscell(Struct_IN.(akt_Feld)),
                Feld_als_Cell = Struct_IN.(akt_Feld)(:); % Mit (:) werden immer Spaltenvektoren generiert (n x 1)
            else
                Feld_als_Cell = num2cell(Struct_IN.(akt_Feld)(:)); % Matrizen mssen als Vektor geschrieben werden, daher (:)
            end
            
            % Da "Feld_als_Cell" immer ein Spaltenvektor ist, muss bei "cell2struct" die Dimension der Cell als 2 angegeben werden:
            DIM = 2;
            
            %% Aus der Cell wird eine mehrdimensionale Struct:
            Struct_Feld_mehrdimensional = cell2struct(Feld_als_Cell, 'Feld', DIM);
            
            Anzahl_Elemente_Feld = size(Struct_Feld_mehrdimensional, 1);
            
            
            % Diese Struct wird bergeben:
            [Struct_AUS(1 : Anzahl_Elemente_Feld).(akt_Feld)] = Struct_Feld_mehrdimensional.Feld;
        end
        
    case 2, % NEU: Direkter Weg.
        for cnt_F = 1 : length(Felder),
            akt_Feld = Felder{cnt_F};
            % ndern von einem Feld:
            % Feld als Cell schreiben:
            if iscell(Struct_IN.(akt_Feld)),
                Feld_als_Cell = Struct_IN.(akt_Feld)(:); % Mit (:) werden immer Spaltenvektoren generiert (n x 1)
            else
                Feld_als_Cell = num2cell(Struct_IN.(akt_Feld)(:)); % Matrizen mssen als Vektor geschrieben werden, daher (:)
            end
            
            Anzahl_Elemente_Feld = size(Feld_als_Cell, 1);
            
            % Diese Struct wird bergeben:
            [Struct_AUS(1 : Anzahl_Elemente_Feld).(akt_Feld)] = Feld_als_Cell{:};
        end        
end

















