function [idx_Zeile, Anzahl_gleiche_Folgeintervalle] = Vektor_gleiche_Folgewerte(Vektor)
% Gibt einen Vektor zurck mit den Zeilen, fr welche die Nachfolgenden Elemente identisch sind.
% 
% ber die letzte Zeile kann keine Aussage getroffen werden, ob sie mit dem folgenden gleich ist.
% Es wird idx_Zeile(end) = false; gesetzt.
%
% NaN und Inf werden NICHT als gleiche Werte betrachtet (vgl. Beispiel #3).
%
%    15.10.2013   JL
%
% % Beispiel #1:
% Vektor = [1 2 3 3 3 4 4 5 6 7 7 7 7 7 8 9 10]';
% [idx_Zeile, Anzahl_gleiche_Folgeintervalle] = Vektor_gleiche_Folgewerte(Vektor)
% [Vektor, idx_Zeile, Anzahl_gleiche_Folgeintervalle]
%
% % Beispiel #2:
% Vektor = [1 1 2 2 3 4 4 4]';
% [idx_Zeile, Anzahl_gleiche_Folgeintervalle] = Vektor_gleiche_Folgewerte(Vektor)
% [Vektor, idx_Zeile, Anzahl_gleiche_Folgeintervalle]
%
% % Beispiel #3:
% Vektor = [1 1 nan nan nan 4 4 inf inf inf]';
% [idx_Zeile, Anzahl_gleiche_Folgeintervalle] = Vektor_gleiche_Folgewerte(Vektor)
% [Vektor, idx_Zeile, Anzahl_gleiche_Folgeintervalle]

% Nchsten Element ist identisch mit dem aktuellen Element: 
Gleiche_Werte = diff(Vektor) == 0;

% Die Stellen finden, bei denen "Gleiche_Werte" vom vorherigen Element NICHT gleich dem aktuellen ist UND das
% nachfolgende dem aktuellen GLEICH ist. Das sind die Zeilen, in denen die Werte zum ersten mal gleich sind.
idx_Zeile = false(size(Vektor)); % vordimensionieren
% Fr die erste Zeile muss nicht geprft werden, ob es zum vorhergehenden NICHT gleich ist:
idx_Zeile(1) = Gleiche_Werte(1);
% ber die letzte Zeile kann keine Aussage getroffen werden, ob sie mit dem folgenden gleich ist.
% Es wird idx_Zeile(end) = false; gesetzt.
idx_Zeile(end) = false;
idx_Zeile(2 : end - 1) = ~Gleiche_Werte(1 : end - 1) & Gleiche_Werte(2 : end);


%% Anzahl_gleiche_Folgeintervalle
if nargout > 1,
    Anzahl_Elemente = numel(Vektor);
    Anzahl_gleiche_Folgeintervalle = zeros(size(Vektor)); % vordimensionieren
    
    % Zeilen an denen gleiche Werte folgen:
    Zeilen_beginn_gleiche_Werte = find(idx_Zeile);
    for cnt_ZbgW = 1 : length(Zeilen_beginn_gleiche_Werte),
        akt_Zeile = Zeilen_beginn_gleiche_Werte(cnt_ZbgW);
        Anzahl_gleiche_Folgeintervalle(akt_Zeile) = 2; 
        % Prfen ob das nchste Element auch noch gleich ist:
        cnt = 2; % das nchste Element ist bereits gleich mit dem ersten, daher wird mit dem bernchsten begonnen.
        while akt_Zeile + cnt <= Anzahl_Elemente && Vektor(akt_Zeile) == Vektor(akt_Zeile + cnt),
            cnt = cnt + 1;
            Anzahl_gleiche_Folgeintervalle(akt_Zeile) = Anzahl_gleiche_Folgeintervalle(akt_Zeile) + 1;
        end
    end
end


end % Main function
