function Zeit_aus = unix_Zeit ( Zeit_ein, welche_Richtung, datetick_format )
%
% Zeit_aus = unix_Zeit ( Zeit_ein, welche_Richtung, datetick_format )
%
% Berechnet aus der Unix Zeit die Matlab Zeit oder umgekehrt.
% Sommer / Winterzeit wird bercksichtigt. 
%
% Bei der Umstellung von Sommerzeit (MESZ) auf Winterzeit (MEZ) gibt es die Tagesstunde 02:00 - 03:00 doppelt.
% Die Funktion kann anhand der Matlab-Zeit nicht unterschieden, ob es die erste Stunde von 02:00 - 03:00 Uhr war oder die zweite.
% Die Unix-Zeit gibt IMMER die zweite Stunde aus !!!!
%
% Beispiel: Zeitumstellung von SZ auf WZ am 30.11.2011 03:00 Uhr.
% num2str(unix_Zeit(datenum('10-30-2011 01:59:59'))) => 1319932799
% num2str(unix_Zeit(datenum('10-30-2011 02:00:00'))) => 1319936400 (Sprung von 3600s)
%
% Eingnge:
%  - Zeit_ein (Zeit als Matlab Zeit ODER Unix-Zeit)
%  - welche_Richtung (1: Von Matlab-Zeit zu Unix-Zeit, sonst: Von Unix zu Matlab-Zeit, 3: von Unix-Zeit zu String (dd.mm.yyyy HH:MM:SS))
%
% Ausgabe:
%  - Zeit_aus (in Unix bzw. Matlab-Zeit Format).
%
% Bentigte externe Funktionen:
%  - Sommerzeit

if nargin < 2 || isempty(welche_Richtung),
    % bestimmen, welche Zeit eingegeben wurde:
    if sum(Zeit_ein > 1e6 | Zeit_ein < 0) > length(Zeit_ein) / 2
        % Wenn der mehr als die Hlfte der Zeit-Werte grer als 1.000.000 oder kleiner als 0 sind, dann ist es eine Unix-Zeit.
        % Unix-Zeit 1.000.000   => 11.01.1970
        % Matlab-Zeit 1.000.000 => 27.11.2737
        welche_Richtung = 0;
    else
        welche_Richtung = 1;
    end
end

% Matlab Zeit am 01.01.1970 um 00:00 Uhr (= Unix Zeit 0)
matlab_zeit_1970 = 719529;

if welche_Richtung == 1, 
    % Umrechnung von Matlab-Zeit in Unix-Zeit.
  
    % Matlab Zeit auf UTC umstellen. Bei Winterzeit wird eine Stunde, bei Sommerzeit 2 Stunden abgezogen.
    Zeit_UTC = Zeit_ein - 1/24 - (Sommerzeit(Zeit_ein) * 1/24); 
    
    
    % Mitteleuropische Unix Zeit:
     Zeit_aus = (Zeit_UTC - matlab_zeit_1970) * 86400;  % 86400 ist die Zahl der Sekunden an einem Tag (24*60*60).
   
else
    
    % Von der Unix-Zeit zur Matlab Zeit:

    % UTC Unix Zeit: 
    Zeit_aus_UTC = matlab_zeit_1970 + Zeit_ein /60/60/24;
    
    % MEZ Zeit (UTC +1) Winterzeit
    Zeit_MEZ = Zeit_aus_UTC + 1/24;
    
    % Bei der Sommerzeit muss eine Stunde zustzlich addiert werden: MESZ (UTC +2)
    Zeit_aus = Zeit_MEZ + (Sommerzeit(Zeit_MEZ) * 1/24); % Zeit_aus_UTC + 1/24 == Winterzeit (MEZ)
    
end

% Zeit als Sting: 
if welche_Richtung == 3, 
    if nargin < 3 || isempty(datetick_format), datetick_format = 'dd.mm.yyyy HH:MM:SS'; end
    Zeit_aus = datestr(Zeit_aus, datetick_format);
end
















